* Encoding: windows-1252.
*
* SPSS Command Syntax File
* Re- construction of sample design variables for EU-SILC UDB 2010, version6
*
* EU-SILC D-File (household register):
* UDB_c10D_ver 2010-6 from 01-03-15.csv
*
* For transforming the EU-SILC CSV-data (as released by Eurostat) into SPSS-data (*.sav),
* please use the corresponding SPSS Command Syntax File published at 
* http://www.gesis.org/missy/eu/setups/EU-SILC
*
* (c) GESIS 07/05/2018
* GESIS - Leibniz Institute for the Social Sciences
* German Microdata Lab 
* Anika Herter; Heike Wirth
* http://www.gesis.org/en/institute/
*
* Contact: heike.wirth@gesis.org
* 
**************************************************************************************************************
* AIM AND PROCEDURE 
*
*    The aim of the SPSS-syntax is the reconstruction of original sample design variables psu1 
*    (primary sampling unit) and strata1 (primary strata) in the D-file of the EU-SILC cross-sectional 
*    data. They are necessary to compute standard errors but incomplete in the UDB.
*    The reconstruction procedure has been implemented and published for stata by Goedeme for 
*     waves 2005-2011 and by Goedeme and Trinidade for waves 2012-2014. Goedeme and Trinidade 
*    also documented their procedure in two articles (Goedeme 2013; Trinidade & Goedeme 2016). 
*    The structure of the SPSS-Syntax at hand follows mostly the structure of the stata codes from 
*    Trinidade & Goedeme. 
*    The first step is some data preparation where variable DB020 is renamed as country and
*     DB030 as hid. Due to anonymization, the stratification variable is not at all available in the 
*     UDB. The psu variable DB060 is missing for several countries. Therefore, these countries must 
*     be handled individually. 
*    For creating the stratification variable, it is necessary to compute a region variable for countries 
*    with regional stratification (same countries as above) based on DB040. For all other countries, 
*    region is set to zero. Strata0 is then computed by combining a numeric country code and the 
*    previously constructed region variable. Countries with self-representing psus also have to be 
*    treated individually. A psutest-variable is created based on the psu-variable of the UDB (DB060). 
*    For constructing the psu variable, this variable is combined with the stratification variable. For 
*    countries with missing values for this variable, the household id is alternatively used. Some 
*    country-specific modifications are necessary here. The variable checker is constructed to identify 
*    cases where households moved between selection to be a respondent and the interview itself. 
*    Regrouping of psus within regions is done for these cases.
*    Finally, all not required auxiliary variables are dropped and sample design variables are reviewed.
*    SPSS Complex Samples is used to define the sampling plan (csplan) and to choose desired statistical analysis 
*    (e.g. frequencies).

* For details on EU Silc standard error calculation and stata codes please visit the homepage of Tim Goedeme  
* https://timgoedeme.com/eu-silc-standard-errors/
* 
* Goedeme, T. (2013). How much Confidence can we have in EU-SILC? Complex Sample Designs and the
*     Standard Error of the Europe 2020 Poverty Indicators in Social Indicators Research, 110(1): 89-110,                                                                                         
*     doi:10.1007/s11205-011-9918-2.
* Zardo Trindade, L. and Goedeme, T. (2016) Notes on updating the EU-SILC UDB sample design variables 2012-2014,
*       CSB Working Paper 16/02, Antwerp: Herman Deleeck Centre for Social Policy, University of Antwerp.
*
* Please note: Since the SPSS syntax is based on the STATA syntax of Tim Goedeme 
* (Download: https://timgoedeme.com/eu-silc-standard-errors/), we have also adopted the corresponding comments from him
* (marked as "Goedeme 2011")

*
**************************************************************************************************************
* Open EU Silc D-file.

set unicode no.

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* The following command should contain the complete path and
  name of the SPSS file, usual file extension "sav";
  Change SPSS_FILENAME to your filename .

GET
  FILE='SPSS_FILENAME'.

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* Please note:
* Until line 635 (Complex Sample Definition), there should be probably nothing to change. 
*  
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .


set decimal dot.

**************************************************************************************************************
* 1. Preparation
**************************************************************************************************************
* Generate identifier.
 COMPUTE id=$CASENUM.
  FORMAT id (F8.0).
  EXECUTE.

*generate country and hid variable.
rename variables (DB020 = country). 
execute.
autorecode country /into countrynr.
execute.
rename variables (DB030 = hid).                                                             
                                                   
* Count number of countries.
compute numcountry = 1.
if (country eq lag(country)) numcountry = 0.
execute.
frequencies numcountry.

* show country labels.
temporary.
select if numcountry = 1.
list country .

***************************************************************************************************************
* 2. Special cases that have to be handled before rest
**************************************************************************************************************

compute  psutest = DB060.                     
                                             

**************************************************************************************************************
* 5.1 CZ: "DB060 not uniqe across panels" " (Goedeme 2011: 117).
**************************************************************************************************************.
if (country='CZ') psutest = DB060*10 + DB075 .

**************************************************************************************************************
* 5.3 IT
**************************************************************************************************************
* "Two stage sample design, rotation at PSU level. Large municipalities are self-representing and
*    remain always in the sample
*    -> detect DB060 appearing in at least three out of four panels DB075, assume these are self-representing
*    -> DB062 is filled, but if made unique by DB060, simply acts as a household identifier (as many hid 
*        as unique DB062)" (Goedeme 2011: 125-127).

recode DB060 DB075 (sysmis=-9).
compute constant=1.
execute.
sort cases by country DB060 DB075 hid.
                                      
do if DB060 eq LAG(DB060) and DB075 eq LAG(DB075) .  
    compute constant = 0.
 else if (DB060 ~= lag(DB060) or DB075 ~= lag(DB075)).
    compute constant = 1.
end if. 
execute.
freq constant.
                                                                                                                                                                               
crosstabs country by constant.

SORT CASES BY country DB060.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  country DB060 
   / npanels=sum(constant).
execute.
sort cases by country DB060.
freq npanels.
crosstabs country by npanels.

temporary. 
select if (country='IT').
frequencies npanels.                                                                                 

if ((npanels=4 or npanels=3) and country='IT') psutest = DB062 + 0.1.    
execute.

do if (npanels=4 or npanels=3) and country='IT'.
    compute tester=DB060.
end if.
execute.
recode tester (sysmis = 999999).
sort cases by tester .
compute groupsit = 1.
do if (tester = lag(tester)).
  compute groupsit= lag(groupsit).
else if (tester ne lag(tester)).
 compute groupsit = lag(groupsit) + 1. 
end if.
execute.
recode tester (999999=sysmis).
execute.

**************************************************************************************************************
* 5.4 FR
**************************************************************************************************************.

do if ((DB075=6) AND country='FR').
     recode psutest (else=sysmis).
end if.
execute.
compute constant = 1.

sort cases country DB060.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  country DB060
   / poppsu=sum(DB090).
execute.
descriptives poppsu.

do if  (country ne 'FR') or (country = 'FR' and misval(DB062) ) or (country='FR' and DB075=6).
     recode poppsu (ELSE= sysmis).
end if.
recode poppsu (sysmis = -9).
execute.
 sort cases by poppsu (D). 
compute groups = 1.
do if (poppsu = lag(poppsu)).
     compute groups= lag(groups).
else if (poppsu ne lag(poppsu)).
      compute groups = lag(groups) + 1. 
end if.
execute.
MISSING VALUES poppsu (-9).
 

recode DB062   (sysmis = -999999).
execute.                                                                                                                                                                                     
temporary. 
select if ((groups<=53) and ( DB062 ne -999999 )).                                                                                                                                                                  
FREQUENCIES DB060.  
recode DB062   ( -999999 = sysmis).


do if (groups <=53 and country = 'FR').
           compute psutest2 = DB062 + 0.1.
           recode psutest2 (ELSE=Copy) INTO psutest. 
end if.
execute.

frequencies constant.
descriptives poppsu groups.

**************************************************************************************************************
* 5.5 UK
***************************************************************************************************************
* "Northern Ireland is a self-representing PSU
*  The self-representing PSU (Northern Ireland) is recognisable as the PSU with the largest number 
*  of households, the only PSU which appears in the 4 rotational panels, the PSU with the largest 
*  number of households and the only PSU with missing values for DB070. 
*  self-representing PSU is itself a stratum & PSUs within this stratum are households
*  In EU-SILC 2011, version 1, Northern Ireland is also recognisable as DB040=="UKN0" "
*   (Goedeme 2011: 148-151).

do if country='UK'.
   compute cons=1.
end if.
execute.

missing values constant (-9).

temporary.
select if (country = 'UK').

AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  country DB060 
   / nrpsu=sum(cons).
execute.
frequencies nrpsu.

temporary. 
select if (country = 'UK') .
DESCRIPTIVES nrpsu.   

sort cases country nrpsu.
temporary.
select if (country = 'UK').

AGGREGATE OUTFILE=* MODE=ADDVARIABLES 
   / PRESORTED
   / BREAK   country
   / test0=max(nrpsu).
execute.
freq test0.

temporary. 
select if (country = 'UK').
frequencies npanels.    

temporary. 
select if ((country = 'UK') AND  (npanels = 4)).
DESCRIPTIVES DB060.  

* construction of test1 *.
sort cases country npanels.
temporary.
select if (country = 'UK' & npanels = 4).
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  DB060 npanels
   / test1 = min(DB060).
execute.
freq test1.
temporary. 
select if ((country = 'UK') AND  (nrpsu = test0)).
DESCRIPTIVES DB060.  

* construction of test2 *.
sort cases country nrpsu.
temporary.
select if (country = "UK" & (nrpsu = test0)).
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  DB060 npanels
   / test2 = min(DB060).
execute.
freq test2.

* construction of test3 *.
sort cases country DB070.
temporary.
select if (country = "UK" & (MISVAL(DB070))).
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK  DB060 npanels
   / test3 = min(DB060).
execute.
freq test3.
*.
do if test1 ~= test2 OR test1 ~= test3.
     compute problem=1.
  else if test1 = test2 AND test1 = test3.
    compute problem=0.
end if.
value label  problem       0 'no problem with finding Northern Ireland'
                                    1 'There is a problem with finding Northern Ireland'.
execute.
freq problem.

do if  test1 = test2 AND test1 = test3 AND npanels = 4 AND country = 'UK'.
    recode psutest (-9999 thru 9999 = sysmis).
end if.
execute.

**************************************************************************************************************
* 3. Prepare Stratification variable
**************************************************************************************************************
* "AT: DB060 completely missing - therefore also stratification ignored...
*  Also for BG and PL DB060 is not sufficiently reliable -> switch to assuming a simple random 
*  sample of households (probably under-estimation of the variance)" (Goedeme 2011: 182-183).

frequencies country.
string region0 (A6).
execute.
compute region0 = DB040.
execute.

* "Melilla (ES64) and Ceuta (ES63) must be grouped together as they are part of the same 
*   stratum."  (Goedeme 2011: 192).
recode region0 ('ES63' 'ES64' = 'ES80').  
                                                     
temporary.
select if ( country= 'BE' OR country='CZ' OR country='BG'  OR country='ES'  OR country='FR'  OR country='IT'  OR country='PL' OR country='RO').
autorecode region0 /into region0nr.
execute.
compute region1=region0nr.
recode region1 (sysmis=0).

execute.
descriptives region1.    
FREQUENCIES region1.                               



compute kons=1.
sort cases region1.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
  / presorted 
  / BREAK  kons
  / regiomin = MAX(region1).
execute.
freq regiomin.


     
do if (groups<=53) AND (country='FR').
      COMPUTE region2 = groups + regiomin.
 else if (groups>53) AND (country='FR').
      compute region2= region1.
end if .  
execute.  

temporary.
select if (country = "FR").
descriptives region1 region2.


do if (country = "FR").
 compute region1 = region2.
end if. 

temporary.
select if (country = "FR").
descriptives region1 region2.

temporary.
select if (country ne "FR").
descriptives region1 region2.



delete variables regiomin.
compute kons=1.
sort cases region1.
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
  / presorted 
  / BREAK  kons
  / regiomin = MAX(region1).
execute.
freq regiomin.

                                                                                                                                               
do if (npanels=4 or npanels=3) AND (country='IT').
    COMPUTE region1a = groupsit + regiomin.
 else if (npanels ne 4 & npanels ne 3) AND (country='IT').
    compute region1a= region1.
end if .  
execute. 

temporary.
select if (country = "IT").
descriptives region1 region1a.

do if (country = "IT").
 compute region1 = region1a.
end if. 

temporary.
select if (country = "IT").
descriptives region1 region1a.

temporary.
select if (country ne "IT").
descriptives region1 .

compute strata0 = countrynr*1000 + region1.
descriptives strata0.

temporary. 
select if (country='UK').
descriptives  strata0. 

sort cases country strata0.
temporary.
select if (country = "UK").
                                                                                                                                                                                 
AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / PRESORTED
   / BREAK kons
   / strat1=max(strata0).
execute.
compute stratum=strat1 + 2.
freq stratum.                                                                                                                                                                             
                                                                                                   
do if country='UK' AND MISSING(psutest).
    RECODE stratum (ELSE=Copy) INTO strata0.   
end if .                                                                                                                   
EXECUTE.
DESCRIPTIVES strata0.


**************************************************************************************************************
* 4. Prepare PSU variable
**************************************************************************************************************

DESCRIPTIVES hid.

AGGREGATE OUTFILE=* MODE=ADDVARIABLES
   / BREAK kons
   / minimum1=max(hid).

execute.
freq minimum1.                                                                    
compute maximum1=10.
do if maximum1 <= minimum1.
   recode maximum1 (10=100).
end if.

DESCRIPTIVES psutest.

COMPUTE psu0=strata0 * 1000000  + hid / 100000.
EXECUTE.                                                                                                                     
DESCRIPTIVES psu0.

recode psutest (sysmis=-9999).
execute.
do if (psutest~=-9999).                                                                                                                         
    compute psu0 = strata0 * 1000000 + psutest.                                                               
end if.
MISSING VALUES psutest (-9999).
DESCRIPTIVES psu0.

**************************************************************************************************************
* 5. RE-grouping of PSUs
**************************************************************************************************************
* "(AT), BE, CZ, ES, FR, HU(?), IT: re-group split PSUs!
*  In the case of several countries, stratification by DB040 causes PSUs to be split across regions 
*  because of households moving between moment of selection and moment of interview. Hence, 
*  households that have moved, should be re-allocated to the correct stratum please note that in the
*  case of Poland, PSU codes are not unique across strata and therefore should split after stratification,
*  re-grouping would do more harm than good in other countries for which DB060!=., no households 
*  have moved between moment of selection and moment of interview."  (Goedeme 2011: 192).

compute checker=1.
recode checker (1=sysmis).
execute.
sort cases by country psutest hid .

recode psutest (sysmis=-9999).

vector nocheck(1) .
execute.
rename variables nocheck1=nocheck.
do if (( psutest=-9999)OR ((npanels=4 or npanels=3) AND country='IT') OR (country='FR' AND groups<=53)).
    recode nocheck (sysmis=1).
end if.
execute.
recode psutest (-9999=sysmis).

recode nocheck  (sysmis = -9999).
frequencies nocheck.

missing values psutest ().
if ((psu0 ne LAG(psu0)) and (psutest ne lag(psutest)) OR nocheck=1)  checker=0.
execute.
if ((LAG(psu0) EQ psu0) and (LAG(psutest) eq psutest) and nocheck ne 1) checker=0.
execute.
if ((LAG(psu0) ne psu0 ) and (LAG(psutest) eq psutest ) and nocheck ne 1 ) checker=1.
execute.
if ((LAG(psu0) eq psu0) and (LAG(psutest) ne psutest ) and nocheck ne 1) checker=2.
execute.
freq checker.

missing values psutest (-9999).
missing values nocheck (-9999).
frequencies nocheck.

crosstabs country by  checker.                                                                                                                                          

* reset checker to 0 if PSUs must be split across strata.
do if (country ~= 'BE' and country~='CZ' and country~='ES' and country~='FR' and country~= 'IT').
    RECODE checker (SYSMIS=0) (1=0) (2=0).
end if.
EXECUTE.
crosstabs country by checker.

sort cases by country psu0 id. 

compute auswahl=0.
do if (country='BE' or country='CZ' or country='ES' or country='FR' or country= 'IT') and (psu0~=LAG(psu0)) .
    recode auswahl (0=1).
end if.
filter by auswahl.
crosstabs country by checker.
filter off.

SORT CASES  BY country.                                                                              
Temporary.
SPLIT FILE SEPARATE BY country.
SELECT IF ( checker = 1).
FREQUENCIES VARIABLES=psutest.
SPLIT FILE off.
Execute.

compute key=0.
sort cases country psutest strata0  .
AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
  / PRESORTED
  / BREAK = country psutest strata0
  / strata0freq = N(strata0).

AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES 
 / PRESORTED
 / BREAK = country psutest 
 / aux1 = MAX( strata0freq )
 / aux2 = first(strata0)
 / aux3 = sd(strata0freq).
execute. 

IF (strata0freq = aux1) key = 1.
execute.
compute strata0_mod = key*strata0.

execute.
AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
 / PRESORTED
 / BREAK = country psutest 
 / checkmax = max(checker).
recode checkmax (sysmis=0).

AGGREGATE OUTFILE = * MODE=ADDVARIABLES OVERWRITE=YES
 / PRESORTED
 / BREAK = country psutest 
 / strata_01 = max(strata0_mod).

fre checkmax.
do if (checkmax=0).
  compute strata1=strata0.
end if.

do if (checkmax = 1).
  compute strata1 = strata_01.
end if.

do if (aux3=0 & checkmax=1).
  compute strata1 = aux2.
end if.
execute. 
 
descriptives strata1 psutest psu0.
 
missing values psutest ().
compute psu1=psu0.
execute.
do if (psutest ne -9999).
  compute psu1 =strata1*1000000+psutest.
end if.
execute.
missing values psutest (-9999).
descriptives psu1 psutest psu0 strata1.

DELETE VARIABLES countrynr psutest npanels tester groupsit cons nrpsu region0 region1 strata0 
          psu0 checker nocheck id numcountry constant test0 test1 test2 test3 problem region0nr kons
          regiomin region1a strat1 stratum minimum1 maximum1 auswahl key strata0freq aux1 aux2 
          aux3 strata0_mod checkmax strata_01.

 **************************************************************************************************************
* 6. Finalisation
**************************************************************************************************************
                                                                                                                                                                                                                                           
***   DEFINITION OF THE SAMPLE-PLAN   
* Assumptions (Approximations):
*     One-stage stratified random sample
*     Stratification: Stratification variable (strata1)
*      -  1st stage: Primary sampling units (psu1):
*     Unrestricted random sample on each selection stage
                  
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - .
* The following command should contain the complete path and
*  name your csaplan-file
* change 'CSAPLAN_FILENAME (extension .csaplan) to your filename .             
                                                                                                                                                                                                                   .
CSPLAN ANALYSIS
  /PLAN FILE='CSAPLAN_FILENAME'
  /PLANVARS ANALYSISWEIGHT=DB090       
  /SRSESTIMATOR TYPE=WOR
  /PRINT PLAN
  /DESIGN STRATA=strata1 CLUSTER=psu1 
  /ESTIMATOR TYPE=WR.
 
* EXAMPLE                                                                                                  .
* Complex Samples Crosstabs.
CSTABULATE
  /PLAN FILE='CSAPLAN_FILENAME'
  /TABLES VARIABLES=country BY DB100
  /CELLS ROWPCT
  /STATISTICS SE CIN(95)
  /MISSING SCOPE=TABLE CLASSMISSING=EXCLUDE.

rename variables (country = DB020). 
rename variables (hid = DB030).
COMMENT BOOKMARK;LINE_NUM=618;ID=1.
